/*
 * Design: ECE 274 - Top Level Design Module for Button Debouncer Test Circuit
 * Author: Roman Lysecky 
 * Copyright 2008, All Rights Reserved
 *
 * Date: March 25, 2008
 *
 */

`timescale 1ns / 1ns
module Top(Clk, Rst, DivRst, ButtonIn, LED, LCD_Data, LCD_E, LCD_RS, LCD_RW); 

   input Clk, Rst, DivRst;
   input ButtonIn;
   output [7:0] LED;
   output [11:8] LCD_Data;
   output LCD_E, LCD_RS, LCD_RW;

   wire Clk1kHz, Clk20kHz;
   wire [12:0] RandomValue;
   wire [9:0] ReactionTime;
   wire LCDOk, LCDFail, LCDUpdate, LCDAck;
   wire Debounced;
   wire Fail;
   
   ClkDiv1kHz ClkDiv1kHz_0 (Clk, DivRst, Clk1kHz);
   ClkDiv20kHz ClkDiv20kHz_0 (Clk, DivRst, Clk20kHz);
   LCDDisplay LCDDisplay_0 (Clk20kHz, Rst, LCDOk, LCDFail, LCDUpdate, LCDAck, LCD_Data, LCD_E, LCD_RS, LCD_RW);
   TesterDisplay TesterDisplay_0(Clk1kHz, Rst, Fail, LCDOk, LCDFail,  LCDUpdate, LCDAck);

   DebounceTester DebounceTester_0(Clk, Rst, Debounced, Fail);
   Debouncer Debouncer_0(Clk1kHz, Rst, ButtonIn, Debounced);

   assign LED = {8{Debounced}};
   
endmodule
